@@ -1,7 +1,7 @@
 // qxgeditAbout.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,8 +27,8 @@
 #define QXGEDIT_TITLE       PACKAGE_NAME
 
 #define QXGEDIT_SUBTITLE    "Qt XG Editor"
-#define QXGEDIT_WEBSITE     "http://qxgedit.sourceforge.net"
-#define QXGEDIT_COPYRIGHT   "Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved."
+#define QXGEDIT_WEBSITE     "https://qxgedit.sourceforge.io"
+#define QXGEDIT_COPYRIGHT   "Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved."
 
 #define QXGEDIT_DOMAIN      "rncbc.org"
 
