@@ -1,7 +1,7 @@
 // qxgeditMainForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -52,7 +52,7 @@
 
 #include <QStyleFactory>
 
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QMimeData>
 #endif
 
@@ -64,29 +64,40 @@
 
 #include <QSocketNotifier>
 
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-
 #include <signal.h>
 
 // File descriptor for SIGUSR1 notifier.
-static int g_fdUsr1[2];
+static int g_fdSigusr1[2] = { -1, -1 };
 
 // Unix SIGUSR1 signal handler.
 static void qxgedit_sigusr1_handler ( int /* signo */ )
 {
 	char c = 1;
 
-	(::write(g_fdUsr1[0], &c, sizeof(c)) > 0);
+	(::write(g_fdSigusr1[0], &c, sizeof(c)) > 0);
 }
 
-#endif
+// File descriptor for SIGTERM notifier.
+static int g_fdSigterm[2] = { -1, -1 };
+
+// Unix SIGTERM signal handler.
+static void qxgedit_sigterm_handler ( int /* signo */ )
+{
+	char c = 1;
+
+	(::write(g_fdSigterm[0], &c, sizeof(c)) > 0);
+}
+
+#endif	// HAVE_SIGNAL_H
 
 
 //-------------------------------------------------------------------------
 // qxgeditMainForm -- UI wrapper form.
 //
-qxgeditMainForm *qxgeditMainForm::g_pMainForm = NULL;
+qxgeditMainForm *qxgeditMainForm::g_pMainForm = nullptr;
 
 // Constructor.
 qxgeditMainForm::qxgeditMainForm (
@@ -100,9 +111,9 @@ qxgeditMainForm::qxgeditMainForm (
 	g_pMainForm = this;
 
 	// Initialize some pointer references.
-	m_pOptions = NULL;
-	m_pMidiDevice = NULL;
-	m_pMasterMap = NULL;
+	m_pOptions = nullptr;
+	m_pMidiDevice = nullptr;
+	m_pMasterMap = nullptr;
 
 	// We'll start clean.
 	m_iUntitled   = 0;
@@ -122,25 +133,48 @@ qxgeditMainForm::qxgeditMainForm (
 	// LADISH Level 1 suport.
 	
 	// Initialize file descriptors for SIGUSR1 socket notifier.
-	::socketpair(AF_UNIX, SOCK_STREAM, 0, g_fdUsr1);
-	m_pUsr1Notifier
-		= new QSocketNotifier(g_fdUsr1[1], QSocketNotifier::Read, this);
+	::socketpair(AF_UNIX, SOCK_STREAM, 0, g_fdSigusr1);
+	m_pSigusr1Notifier
+		= new QSocketNotifier(g_fdSigusr1[1], QSocketNotifier::Read, this);
 
-	QObject::connect(m_pUsr1Notifier,
+	QObject::connect(m_pSigusr1Notifier,
 		SIGNAL(activated(int)),
 		SLOT(handle_sigusr1()));
 
 	// Install SIGUSR1 signal handler.
-    struct sigaction usr1;
-    usr1.sa_handler = qxgedit_sigusr1_handler;
-    ::sigemptyset(&usr1.sa_mask);
-    usr1.sa_flags = 0;
-    usr1.sa_flags |= SA_RESTART;
-    ::sigaction(SIGUSR1, &usr1, NULL);
+	struct sigaction sigusr1;
+	sigusr1.sa_handler = qxgedit_sigusr1_handler;
+	sigemptyset(&sigusr1.sa_mask);
+	sigusr1.sa_flags = 0;
+	sigusr1.sa_flags |= SA_RESTART;
+	::sigaction(SIGUSR1, &sigusr1, nullptr);
+
+	// Initialize file descriptors for SIGTERM socket notifier.
+	::socketpair(AF_UNIX, SOCK_STREAM, 0, g_fdSigterm);
+	m_pSigtermNotifier
+		= new QSocketNotifier(g_fdSigterm[1], QSocketNotifier::Read, this);
+
+	QObject::connect(m_pSigtermNotifier,
+		SIGNAL(activated(int)),
+		SLOT(handle_sigterm()));
+
+	// Install SIGTERM signal handler.
+	struct sigaction sigterm;
+	sigterm.sa_handler = qxgedit_sigterm_handler;
+	sigemptyset(&sigterm.sa_mask);
+	sigterm.sa_flags = 0;
+	sigterm.sa_flags |= SA_RESTART;
+	::sigaction(SIGTERM, &sigterm, nullptr);
+	::sigaction(SIGQUIT, &sigterm, nullptr);
+
+	// Ignore SIGHUP/SIGINT signals.
+	::signal(SIGHUP, SIG_IGN);
+	::signal(SIGINT, SIG_IGN);
 
 #else	// HAVE_SIGNAL_H
 
-	m_pUsr1Notifier = NULL;
+	m_pSigusr1Notifier = nullptr;
+	m_pSigtermNotifier = nullptr;
 	
 #endif	// !HAVE_SIGNAL_H
 
@@ -229,8 +263,10 @@ qxgeditMainForm::qxgeditMainForm (
 qxgeditMainForm::~qxgeditMainForm (void)
 {
 #ifdef HAVE_SIGNAL_H
-	if (m_pUsr1Notifier)
-		delete m_pUsr1Notifier;
+	if (m_pSigusr1Notifier)
+		delete m_pSigusr1Notifier;
+	if (m_pSigtermNotifier)
+		delete m_pSigtermNotifier;
 #endif
 
 	// Free designated devices.
@@ -240,7 +276,7 @@ qxgeditMainForm::~qxgeditMainForm (void)
 		delete m_pMasterMap;
 
 	// Pseudo-singleton reference shut-down.
-	g_pMainForm = NULL;
+	g_pMainForm = nullptr;
 }
 
 
@@ -1212,7 +1248,7 @@ void qxgeditMainForm::closeEvent ( QCloseEvent *pCloseEvent )
 void qxgeditMainForm::dragEnterEvent ( QDragEnterEvent* pDragEnterEvent )
 {
 	// Accept external drags only...
-	if (pDragEnterEvent->source() == NULL
+	if (pDragEnterEvent->source() == nullptr
 		&& pDragEnterEvent->mimeData()->hasUrls()) {
 		pDragEnterEvent->accept();
 	} else {
@@ -1245,13 +1281,26 @@ void qxgeditMainForm::handle_sigusr1 (void)
 
 	char c;
 
-	if (::read(g_fdUsr1[1], &c, sizeof(c)) > 0)
+	if (::read(g_fdSigusr1[1], &c, sizeof(c)) > 0)
 		saveSession(false);
 
 #endif
 }
 
 
+void qxgeditMainForm::handle_sigterm (void)
+{
+#ifdef HAVE_SIGNAL_H
+
+	char c;
+
+	if (::read(g_fdSigterm[1], &c, sizeof(c)) > 0)
+		close();
+
+#endif
+}
+
+
 // RPN Event handler.
 void qxgeditMainForm::rpnReceived (
 	unsigned char ch, unsigned short rpn, unsigned short val )
@@ -1322,16 +1371,16 @@ bool qxgeditMainForm::newSession (void)
 // Open an existing sampler session.
 bool qxgeditMainForm::openSession (void)
 {
-	if (m_pOptions == NULL)
+	if (m_pOptions == nullptr)
 		return false;
 
 	// Ask for the filename to open...
 	QString sFilename;
 
 	const QString sExt("syx");
-	const QString& sTitle  = tr("Open Session") + " - " QXGEDIT_TITLE;
+	const QString& sTitle  = tr("Open Session");
 	const QString& sFilter = tr("Session files (*.%1)").arg(sExt);
-#if 0//QT_VERSION < 0x040400
+#if 0//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 	sFilename = QFileDialog::getOpenFileName(this,
 		sTitle, m_pOptions->sSessionDir, sFilter);
 #else
@@ -1370,7 +1419,7 @@ bool qxgeditMainForm::openSession (void)
 // Save current sampler session with another name.
 bool qxgeditMainForm::saveSession ( bool bPrompt )
 {
-	if (m_pOptions == NULL)
+	if (m_pOptions == nullptr)
 		return false;
 
 	// Suggest a filename, if there's none...
@@ -1382,9 +1431,9 @@ bool qxgeditMainForm::saveSession ( bool bPrompt )
 	if (bPrompt) {
 		// Prompt the guy...
 		const QString sExt("syx");
-		const QString& sTitle  = tr("Save Session") + " - " QXGEDIT_TITLE;
+		const QString& sTitle  = tr("Save Session");
 		const QString& sFilter = tr("Session files (*.%1)").arg(sExt);
-	#if 0//QT_VERSION < 0x040400
+	#if 0//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 		sFilename = QFileDialog::getSaveFileName(this,
 			sTitle, sFilename, sFilter);
 	#else
@@ -1415,7 +1464,7 @@ bool qxgeditMainForm::saveSession ( bool bPrompt )
 			// Check if already exists...
 			if (sFilename != m_sFilename && QFileInfo(sFilename).exists()) {
 				if (QMessageBox::warning(this,
-					tr("Warning") + " - " QXGEDIT_TITLE,
+					tr("Warning"),
 					tr("The file already exists:\n\n"
 					"\"%1\"\n\n"
 					"Do you want to replace it?")
@@ -1439,7 +1488,7 @@ bool qxgeditMainForm::closeSession (void)
 	// Are we dirty enough to prompt it?
 	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("The current session has been changed:\n\n"
 			"\"%1\"\n\n"
 			"Do you want to save the changes?")
@@ -1485,7 +1534,7 @@ bool qxgeditMainForm::loadSessionFile ( const QString& sFilename )
 
 	int iSysex = 0;
 	unsigned short iBuff = 0;
-	unsigned char *pBuff = NULL;
+	unsigned char *pBuff = nullptr;
 	unsigned short i = 0;
 
 	// Read the file....
@@ -1688,7 +1737,7 @@ void qxgeditMainForm::viewToolbar ( bool bOn )
 // Randomize current parameter page view.
 void qxgeditMainForm::viewRandomize (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	float p = 100.0f;
@@ -1734,7 +1783,7 @@ void qxgeditMainForm::viewRandomize (void)
 				break;
 			}
 			if (QMessageBox::warning(this,
-				tr("Warning") + " - " QXGEDIT_TITLE,
+				tr("Warning"),
 				tr("About to randomize current parameter values:\n\n"
 				"%1.\n\n"
 				"Are you sure?").arg(sText),
@@ -1783,7 +1832,7 @@ void qxgeditMainForm::viewRandomize (void)
 // Show options dialog.
 void qxgeditMainForm::viewOptions (void)
 {
-	if (m_pOptions == NULL)
+	if (m_pOptions == nullptr)
 		return;
 
 	// Load the current setup settings.
@@ -1824,7 +1873,7 @@ void qxgeditMainForm::viewOptions (void)
 			}
 			// Show restart needed message...
 			QMessageBox::information(this,
-				tr("Information") + " - " QXGEDIT_TITLE,
+				tr("Information"),
 				tr("Some settings may be only effective\n"
 				"next time you start this %1.")
 				.arg(sNeedRestart));
@@ -1864,7 +1913,7 @@ void qxgeditMainForm::helpAbout (void)
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About") + " " QXGEDIT_TITLE, sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -1884,7 +1933,7 @@ void qxgeditMainForm::stabilizeForm (void)
 	QString sSessionName = sessionName(m_sFilename);
 	if (m_iDirtyCount > 0)
 		sSessionName += ' ' + tr("[modified]");
-	setWindowTitle(sSessionName + " - " QXGEDIT_TITLE);
+	setWindowTitle(sSessionName);
 
 	// Update the main menu state...
 	m_ui.fileSaveAction->setEnabled(m_iDirtyCount > 0);
@@ -1914,7 +1963,7 @@ void qxgeditMainForm::stabilizeForm (void)
 // Update the recent files list and menu.
 void qxgeditMainForm::updateRecentFiles ( const QString& sFilename )
 {
-	if (m_pOptions == NULL)
+	if (m_pOptions == nullptr)
 		return;
 
 	// Remove from list if already there (avoid duplicates)
@@ -1929,7 +1978,7 @@ void qxgeditMainForm::updateRecentFiles ( const QString& sFilename )
 // Update the recent files list and menu.
 void qxgeditMainForm::updateRecentFilesMenu (void)
 {
-	if (m_pOptions == NULL)
+	if (m_pOptions == nullptr)
 		return;
 
 	// Time to keep the list under limits.
@@ -1962,12 +2011,12 @@ void qxgeditMainForm::masterReset (void)
 
 void qxgeditMainForm::masterResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"XG System.\n\n"
 			"Are you sure?"),
@@ -1982,12 +2031,12 @@ void qxgeditMainForm::masterResetButtonClicked (void)
 
 void qxgeditMainForm::reverbResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"REVERB / %1.\n\n"
 			"Are you sure?").arg(m_ui.ReverbTypeCombo->currentText()),
@@ -2002,12 +2051,12 @@ void qxgeditMainForm::reverbResetButtonClicked (void)
 
 void qxgeditMainForm::chorusResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"CHORUS / %1.\n\n"
 			"Are you sure?").arg(m_ui.ChorusTypeCombo->currentText()),
@@ -2022,12 +2071,12 @@ void qxgeditMainForm::chorusResetButtonClicked (void)
 
 void qxgeditMainForm::variationResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"VARIATION / %1.\n\n"
 			"Are you sure?").arg(m_ui.VariationTypeCombo->currentText()),
@@ -2224,14 +2273,14 @@ void qxgeditMainForm::multipartPartModeChanged ( unsigned short iPartMode )
 
 void qxgeditMainForm::multipartResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	const int iPart = m_ui.MultipartCombo->currentIndex();
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"MULTI PART / %1.\n\n"
 			"Are you sure?").arg(m_ui.MultipartCombo->currentText()),
@@ -2299,14 +2348,14 @@ void qxgeditMainForm::drumsetupNoteComboActivated ( int iNote )
 
 void qxgeditMainForm::drumsetupResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	const int iDrumset = m_ui.DrumsetupCombo->currentIndex();
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"DRUM SETUP / %1.\n\n"
 			"Are you sure?").arg(m_ui.DrumsetupCombo->currentText()),
@@ -2358,14 +2407,14 @@ void qxgeditMainForm::uservoiceElementComboActivated ( int iElem )
 
 void qxgeditMainForm::uservoiceResetButtonClicked (void)
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return;
 
 	const int iUser = m_ui.UservoiceCombo->currentIndex();
 
 	if (m_pOptions && m_pOptions->bConfirmReset) {
 		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("About to reset all parameters to default:\n\n"
 			"USER VOICE / %1.\n\n"
 			"Are you sure?").arg(m_ui.UservoiceCombo->currentText()),
@@ -2426,7 +2475,7 @@ void qxgeditMainForm::contentsChanged (void)
 // Check whether randomize current parameter page view is possible.
 bool qxgeditMainForm::isRandomizable (void) const
 {
-	if (m_pMasterMap == NULL)
+	if (m_pMasterMap == nullptr)
 		return false;
 
 	switch (m_ui.MainTabWidget->currentIndex()) {
@@ -2505,7 +2554,7 @@ void qxgeditMainForm::uservoiceLoadPresetFile ( const QString& sFilename )
 	if (!bResult) {
 		// Failure (maybe wrong preset)...
 		QMessageBox::critical(this,
-			tr("Error") + " - " QXGEDIT_TITLE,
+			tr("Error"),
 			tr("Preset could not be loaded:\n\n"
 			"\"%1\".\n\n"
 			"Sorry.").arg(sFilename),
@@ -2552,8 +2601,7 @@ void qxgeditMainForm::showMessage( const QString& s )
 
 void qxgeditMainForm::showMessageError( const QString& s )
 {
-	QMessageBox::critical(this,
-		tr("Error") + " - " QXGEDIT_TITLE, s, QMessageBox::Ok);
+	QMessageBox::critical(this, tr("Error"), s, QMessageBox::Ok);
 }
 
 
