@@ -1,7 +1,7 @@
 // qxgeditOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -60,9 +60,10 @@ public:
 	bool    bConfirmReset;
 	bool    bConfirmRemove;
 	bool    bCompletePath;
-	int     iRandomizePerct;
+	float   fRandomizePercent;
 	int     iBaseFontSize;
 	QString sStyleTheme;
+	QString sColorTheme;
 
 	// View options...
 	bool    bMenubar;
