@@ -1,7 +1,7 @@
 // qxgeditOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,13 +19,15 @@
 
 *****************************************************************************/
 
+#include "qxgeditAbout.h"
 #include "qxgeditOptionsForm.h"
 
-#include "qxgeditAbout.h"
 #include "qxgeditOptions.h"
 
 #include "qxgeditMidiDevice.h"
 
+#include "qxgeditPaletteForm.h"
+
 #include <QMessageBox>
 #include <QPushButton>
 #include <QValidator>
@@ -33,6 +35,10 @@
 #include <QStyleFactory>
 
 
+// Default (empty/blank) name.
+static const char *g_pszDefName = QT_TRANSLATE_NOOP("qxgeditOptionsForm", "(default)");
+
+
 //----------------------------------------------------------------------------
 // qxgeditOptionsForm -- UI wrapper form.
 
@@ -45,7 +51,7 @@ qxgeditOptionsForm::qxgeditOptionsForm (
 	m_ui.setupUi(this);
 
 	// No settings descriptor initially (the caller will set it).
-	m_pOptions = NULL;
+	m_pOptions = nullptr;
 
 	// Initialize dirty control state.
 	m_iDirtyCount = 0;
@@ -58,11 +64,6 @@ qxgeditOptionsForm::qxgeditOptionsForm (
 	m_ui.MidiInputListView->setMaximumHeight(72);
 	m_ui.MidiOutputListView->setMaximumHeight(72);
 
-	// Custom style themes...
-	//m_ui.StyleThemeComboBox->clear();
-	//m_ui.StyleThemeComboBox->addItem(tr("(default)"));
-	m_ui.StyleThemeComboBox->addItems(QStyleFactory::keys());
-
 	adjustSize();
 
 	// UI signal/slot connections...
@@ -93,6 +94,12 @@ qxgeditOptionsForm::qxgeditOptionsForm (
 	QObject::connect(m_ui.StyleThemeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.ColorThemeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.ColorThemeToolButton,
+		SIGNAL(clicked()),
+		SLOT(editColorThemes()));
 	QObject::connect(m_ui.DialogButtonBox,
 		SIGNAL(accepted()),
 		SLOT(accept()));
@@ -153,13 +160,9 @@ void qxgeditOptionsForm::setOptions ( qxgeditOptions *pOptions )
 	else
 		m_ui.BaseFontSizeComboBox->setCurrentIndex(0);
 
-	// Custom style theme...
-	int iStyleTheme = 0;
-	if (!m_pOptions->sStyleTheme.isEmpty()) {
-		iStyleTheme = m_ui.StyleThemeComboBox->findText(
-			m_pOptions->sStyleTheme, Qt::MatchExactly);
-	}
-	m_ui.StyleThemeComboBox->setCurrentIndex(iStyleTheme);
+	// Custom display options...
+	resetColorThemes(m_pOptions->sColorTheme);
+	resetStyleThemes(m_pOptions->sStyleTheme);
 
 	// Done. Restart clean.
 	m_iDirtyCount = 0;
@@ -211,11 +214,45 @@ void qxgeditOptionsForm::accept (void)
 		m_pOptions->iMaxRecentFiles = m_ui.MaxRecentFilesSpinBox->value();
 		m_pOptions->fRandomizePercent = float(m_ui.RandomizePercentSpinBox->value());
 		m_pOptions->iBaseFontSize   = m_ui.BaseFontSizeComboBox->currentText().toInt();
-		// Custom style theme...
+		// Custom options...
+		const QString sOldStyleTheme = m_pOptions->sStyleTheme;
 		if (m_ui.StyleThemeComboBox->currentIndex() > 0)
 			m_pOptions->sStyleTheme = m_ui.StyleThemeComboBox->currentText();
 		else
 			m_pOptions->sStyleTheme.clear();
+		const QString sOldColorTheme = m_pOptions->sColorTheme;
+		if (m_ui.ColorThemeComboBox->currentIndex() > 0)
+			m_pOptions->sColorTheme = m_ui.ColorThemeComboBox->currentText();
+		else
+			m_pOptions->sColorTheme.clear();
+		// Check whether restart is needed or whether
+		// custom options maybe set up immediately...
+		int iNeedRestart = 0;
+		if (m_pOptions->sStyleTheme != sOldStyleTheme) {
+			if (m_pOptions->sStyleTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QApplication::setStyle(
+					QStyleFactory::create(m_pOptions->sStyleTheme));
+			}
+		}
+		if (m_pOptions->sColorTheme != sOldColorTheme) {
+			if (m_pOptions->sColorTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QPalette pal;
+				if (qxgeditPaletteForm::namedPalette(
+						&m_pOptions->settings(), m_pOptions->sColorTheme, pal))
+					QApplication::setPalette(pal);
+			}
+		}
+		// Show restart message if needed...
+		if (iNeedRestart > 0) {
+			QMessageBox::information(this,
+				tr("Information"),
+				tr("Some settings may be only effective\n"
+				"next time you start this application."));
+		}
 		// Reset dirty flag.
 		m_iDirtyCount = 0;
 	}
@@ -236,7 +273,7 @@ void qxgeditOptionsForm::reject (void)
 	// Check if there's any pending changes...
 	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
-			tr("Warning") + " - " QXGEDIT_TITLE,
+			tr("Warning"),
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			QMessageBox::Apply |
@@ -264,12 +301,81 @@ void qxgeditOptionsForm::changed (void)
 	stabilizeForm();
 }
 
+
+// Custom color palette theme manager.
+void qxgeditOptionsForm::editColorThemes (void)
+{
+	qxgeditPaletteForm form(this);
+	form.setSettings(&m_pOptions->settings());
+
+	QString sColorTheme;
+	int iDirtyColorTheme = 0;
+
+	const int iColorTheme
+		= m_ui.ColorThemeComboBox->currentIndex();
+	if (iColorTheme > 0) {
+		sColorTheme = m_ui.ColorThemeComboBox->itemText(iColorTheme);
+		form.setPaletteName(sColorTheme);
+	}
+
+	if (form.exec() == QDialog::Accepted) {
+		sColorTheme = form.paletteName();
+		++iDirtyColorTheme;
+	}
+
+	if (iDirtyColorTheme > 0 || form.isDirty()) {
+		resetColorThemes(sColorTheme);
+		changed();
+	}
+}
+
+
+// Custom color palette themes settler.
+void qxgeditOptionsForm::resetColorThemes ( const QString& sColorTheme )
+{
+	m_ui.ColorThemeComboBox->clear();
+	m_ui.ColorThemeComboBox->addItem(
+		tr(g_pszDefName));
+	m_ui.ColorThemeComboBox->addItems(
+		qxgeditPaletteForm::namedPaletteList(&m_pOptions->settings()));
+
+	int iColorTheme = 0;
+	if (!sColorTheme.isEmpty()) {
+		iColorTheme = m_ui.ColorThemeComboBox->findText(sColorTheme);
+		if (iColorTheme < 0)
+			iColorTheme = 0;
+	}
+	m_ui.ColorThemeComboBox->setCurrentIndex(iColorTheme);
+}
+
+
+// Custom widget style themes settler.
+void qxgeditOptionsForm::resetStyleThemes ( const QString& sStyleTheme )
+{
+	m_ui.StyleThemeComboBox->clear();
+	m_ui.StyleThemeComboBox->addItem(
+		tr(g_pszDefName));
+	m_ui.StyleThemeComboBox->addItems(QStyleFactory::keys());
+
+	int iStyleTheme = 0;
+	if (!sStyleTheme.isEmpty()) {
+		iStyleTheme = m_ui.StyleThemeComboBox->findText(sStyleTheme);
+		if (iStyleTheme < 0)
+			iStyleTheme = 0;
+	}
+	m_ui.StyleThemeComboBox->setCurrentIndex(iStyleTheme);
+}
+
+
+
+// MIDI I/O changes
 void qxgeditOptionsForm::midiInputsChanged (void)
 {
 	m_iMidiInputsChanged++;
 	changed();
 }
 
+
 void qxgeditOptionsForm::midiOutputsChanged (void)
 {
 	m_iMidiOutputsChanged++;
