@@ -1,7 +1,7 @@
 // qxgeditOptionsForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -53,11 +53,19 @@ protected slots:
 
 	void changed();
 
+	void editColorThemes();
+
 	void midiInputsChanged();
 	void midiOutputsChanged();
 	
 	void stabilizeForm();
 
+protected:
+
+	// Custom color/style themes settlers.
+	void resetColorThemes(const QString& sColorTheme);
+	void resetStyleThemes(const QString& sStyleTheme);
+
 private:
 
 	// The Qt-designer UI struct...
